
// mod file for Ellison/Tischbirek (JEDC, 2014)
// by Benedikt Kolb, May 2016

parameters 
alf bet del psi phi tau tp f a1 a2 gB gQ 
gamp gamy gqep gqey
PISS tet GYSS
rho_n rho_x rho_c rho_l rho_g rho_a rho_t 
sig_n sig_x sig_c sig_l sig_g sig_a sig_t
;

var 
C    % consumption
PS   % price of overall bonds
S    % overall bonds 
PI   % inflation
W    % real wage
L    % labour supply
Y    % output 
F    % price level component I
K    % price level component I
B    % one-period bonds
Q    % multi-period bonds
PB   % price of one-period bonds
I    % nominal net interest rate
PQ   % price of multi-period bonds
IQ   % nominal net interest rate on multi-period bonds
QB   % supply of multi-period bonds
QCB  % central bank purchases of multi-period bonds
D    % price distortion

SHOCKC  % AR(1) shock to 
SHOCKL  % AR(1) shock to 
SHOCKA  % AR(1) shock to technology
SHOCKT  % AR(1) shock to 
SHOCKG  % AR(1) shock to 
SHOCKN  % AR(1) shock to 
SHOCKX  % AR(1) shock to 

PSS     % (for plotting): PS*S
;

varexo  % shock to ...
e_c     % inter-temporal preferences
e_l     % labour supply
e_a     % technology
e_t     % intra-temporal elasticity of substitution
e_g     % government spending
e_n     % conventional monetary policy
e_x     % unconventional monetary policy
;

alf   = 0.85;   % Degree of price rigidity
bet   = 0.99;   % Household discount factor
del   = 2;      % Inverse elasticity of intertemporal substitution in consumption
tet   = 6;      % Steady-state value of intratemporal elasticity of substitution
psi   = 0.5;    % Inverse Frisch elasticity of labour supply
phi   = 1.1;    % Inverse of returns to scale in production
tau   = 20;     % Horizon of long-term bond
tp    = 0.5;    % Share of firm profits received by the government
f     = 0.66;   % Parameter in long-term bond supply rule
a1    = 0.95;   % Asset demand
a2    = 0;      % Asset demand
gB    = 10.21;  % Asset demand (subsistence level of B)
gQ    = 0.59;   % Asset demand (subsistence level of Q)
gamp  = 1.01;   % Taylor rule weight on inflation
gamy  = 0.3;    % Taylor rule weight on output
gqep  = 0;      % Asset-purchase rule weight on inflation
gqey  = 60;     % Asset-purchase rule weight on output

PISS     = 1.005;  % Steady-state inflation
GYSS     = 0.4;    % Steady-state ratio of government spending to GDP

rho_n = 0.1;    % Persistence of shock to interest rate rule
rho_x = 0.1;    % Persistence of shock to asset purchase rule
rho_c = 0.1;    % Persistence of consumption preference shock
rho_l = 0.7;    % Persistence of labour supply preference shock
rho_g = 0.1;    % Persistence of government spending
rho_a = 0.7;    % Persistence of technology shock
rho_t = 0.95;   % Persistence of shock to elasticity of substitution
sig_n = 0.0025; % Standard deviation of shock to interest rate rule
sig_x = 0.0025; % Standard deviation of shock to asset purchase rule
sig_c = 0.0025; % Standard deviation of consumption preference shock
sig_l = 0.0025; % Standard deviation of labour supply preference shock
sig_g = 0.005;  % Standard deviation of government spending shock
sig_a = 0.01;   % Standard deviation of technology shock
sig_t = 0.06;   % Standard deviation of shock to elasticity of substitution



model;

#DSS = (1-alf)/(1-alf*PISS^(tet*phi))
       *((1-alf*PISS^(tet-1))/(1-alf))^(tet*phi/(tet-1));
#YSS = ((1-alf*bet*PISS^(tet-1))/(1-alf*bet*PISS^(tet*phi))
       *(tet*phi)/(tet-1)*DSS^psi*(1-GYSS)^del
       *((1-alf*PISS^(tet-1))/(1-alf))^((tet*(phi-1)+1)/(tet-1)))
       ^(1/(1-del-phi*(psi+1)));
#CSS  = (1-GYSS)*YSS;
#LSS  = DSS*YSS^phi;
#WSS  = LSS^psi*CSS^del;
#PSSS = bet/PISS;
#QSS  = f*YSS;
#SSS  = PISS/(1-bet)*(CSS-tp*WSS*LSS-(1-tp-GYSS)*YSS);
#BSS  = SSS-QSS/tau*(1-1/PISS^tau)*PISS/(PISS-1);
#PBSS = a1*PSSS*SSS/(BSS-(1-a1)*gB+(1-a1)*(BSS-gB)/(QSS-gQ)*gQ);
#PQSS = (1-a1)/a1*(BSS-gB)/(QSS-gQ)*PBSS;
#ISS  = 1/PBSS-1;


//HOUSEHOLD
// 1. HH budget constraint
exp(C) = exp(S(-1))/exp(PI) + tp*exp(W)*exp(L) + (1-tp)*exp(Y)
         - exp(PS)*exp(S) - exp(SHOCKG);

// 2. Euler equation
exp(PS) = bet*exp(SHOCKC(+1))/exp(SHOCKC)*(exp(C(+1))/exp(C))^(-del)
          /exp(PI(+1));

// 3. Wage
exp(W) = exp(SHOCKL)/exp(SHOCKC)*exp(L)^psi*exp(C)^del;

// 4. Definition inflation
1 = alf*exp(PI)^(exp(SHOCKT)-1) 
    + (1-alf)*(exp(F)/exp(K))^((exp(SHOCKT)-1)/(exp(SHOCKT)*(phi-1)+1));
% typo in the paper (eq. B.4), parenthesis in exponent missing

// 5. Auxiliary inflation term I
exp(F) = exp(SHOCKC)*exp(C)^(-del)*exp(Y) 
         + alf*bet*exp(PI(+1))^(exp(SHOCKT)-1)*exp(F(+1));

// 6.  Auxiliary inflation term II
exp(K) = exp(SHOCKT)*phi/(exp(SHOCKT)-1)*exp(SHOCKL)*exp(L)^psi
         *(exp(Y)/exp(SHOCKA))^phi 
         + alf*bet*exp(PI(+1))^(exp(SHOCKT)*phi)*exp(K(+1));

// 7. Return to household savings
exp(S) = exp(B) + 1/tau*(exp(Q) + exp(Q(-1))/exp(PI)
         + exp(Q(-2)) /exp(PI)  /exp(PI(-1))
         + exp(Q(-3)) /exp(PI)  /exp(PI(-1)) /exp(PI(-2))
         + exp(Q(-4)) /exp(PI)  /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
         + exp(Q(-5)) /exp(PI)  /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
                   /exp(PI(-4))
         + exp(Q(-6)) /exp(PI)  /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
                   /exp(PI(-4)) /exp(PI(-5))
         + exp(Q(-7))/exp(PI)   /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
                   /exp(PI(-4)) /exp(PI(-5)) /exp(PI(-6))
         + exp(Q(-8))/exp(PI)   /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
                   /exp(PI(-4)) /exp(PI(-5)) /exp(PI(-6)) /exp(PI(-7))
         + exp(Q(-9))/exp(PI)   /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
                   /exp(PI(-4)) /exp(PI(-5)) /exp(PI(-6)) /exp(PI(-7))
                   /exp(PI(-8))
         + exp(Q(-10))/exp(PI)  /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
                   /exp(PI(-4)) /exp(PI(-5)) /exp(PI(-6)) /exp(PI(-7))
                   /exp(PI(-8)) /exp(PI(-9))
         + exp(Q(-11))/exp(PI)  /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
                   /exp(PI(-4)) /exp(PI(-5)) /exp(PI(-6)) /exp(PI(-7))
                   /exp(PI(-8)) /exp(PI(-9)) /exp(PI(-10))
         + exp(Q(-12)) /exp(PI) /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
                   /exp(PI(-4)) /exp(PI(-5)) /exp(PI(-6)) /exp(PI(-7))
                   /exp(PI(-8)) /exp(PI(-9)) /exp(PI(-10))/exp(PI(-11))
         + exp(Q(-13)) /exp(PI) /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
                   /exp(PI(-4)) /exp(PI(-5)) /exp(PI(-6)) /exp(PI(-7))
                   /exp(PI(-8)) /exp(PI(-9)) /exp(PI(-10))/exp(PI(-11))
                   /exp(PI(-12))
         + exp(Q(-14)) /exp(PI) /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
                   /exp(PI(-4)) /exp(PI(-5)) /exp(PI(-6)) /exp(PI(-7))
                   /exp(PI(-8)) /exp(PI(-9)) /exp(PI(-10))/exp(PI(-11))
                   /exp(PI(-12))/exp(PI(-13))
         + exp(Q(-15)) /exp(PI) /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
                   /exp(PI(-4)) /exp(PI(-5)) /exp(PI(-6)) /exp(PI(-7))
                   /exp(PI(-8)) /exp(PI(-9)) /exp(PI(-10))/exp(PI(-11))
                   /exp(PI(-12))/exp(PI(-13))/exp(PI(-14))
         + exp(Q(-16)) /exp(PI) /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
                   /exp(PI(-4)) /exp(PI(-5)) /exp(PI(-6)) /exp(PI(-7))
                   /exp(PI(-8)) /exp(PI(-9)) /exp(PI(-10))/exp(PI(-11))
                   /exp(PI(-12))/exp(PI(-13))/exp(PI(-14))/exp(PI(-15))
         + exp(Q(-17)) /exp(PI) /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
                   /exp(PI(-4)) /exp(PI(-5)) /exp(PI(-6)) /exp(PI(-7))
                   /exp(PI(-8)) /exp(PI(-9)) /exp(PI(-10))/exp(PI(-11))
                   /exp(PI(-12))/exp(PI(-13))/exp(PI(-14))/exp(PI(-15))
                   /exp(PI(-16))
         + exp(Q(-18)) /exp(PI) /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
                   /exp(PI(-4)) /exp(PI(-5)) /exp(PI(-6)) /exp(PI(-7))
                   /exp(PI(-8)) /exp(PI(-9)) /exp(PI(-10))/exp(PI(-11))
                   /exp(PI(-12))/exp(PI(-13))/exp(PI(-14))/exp(PI(-15))
                   /exp(PI(-16))/exp(PI(-17))
         + exp(Q(-19)) /exp(PI) /exp(PI(-1)) /exp(PI(-2)) /exp(PI(-3))
                   /exp(PI(-4)) /exp(PI(-5)) /exp(PI(-6)) /exp(PI(-7))
                   /exp(PI(-8)) /exp(PI(-9)) /exp(PI(-10))/exp(PI(-11))
                   /exp(PI(-12))/exp(PI(-13))/exp(PI(-14))/exp(PI(-15))
                   /exp(PI(-16))/exp(PI(-17))/exp(PI(-18))
         );

// 8. Definition nominal interest rate
1 = 1/exp(PB) - exp(I);

// 9. (Implicit) Definition of interest rate on multi-period bond
exp(PQ) = (1-(1/(1+exp(IQ)))^tau)/tau/(1+exp(IQ))/(1-1/(1+exp(IQ)));

//10. Demand for one-period bond
exp(B) = gB + (exp(PS)*exp(S)-exp(PB)*gB-exp(PQ)*gQ)/exp(PB)
               *  (a1+a2*log(exp(PB)/exp(PQ)));

//11. Demand for multi-period bond
exp(Q) = gQ + (exp(PS)*exp(S)-exp(PB)*gB-exp(PQ)*gQ)/exp(PQ)
               *(1-a1-a2*log(exp(PB)/exp(PQ)));

//12. Supply of multi-period bonds
exp(QB) = f*YSS;

//13. Taylor rule
(1+exp(I))/(1+ISS) = (exp(PI)/PISS)^gamp*(exp(Y)/YSS)^gamy*exp(SHOCKN);

//14. Asset-purchasing rule
(exp(QB)-QCB)/exp(QB) = (exp(PI)/PISS)^gqep*(exp(Y)/YSS)^gqey*exp(SHOCKX);

//15. Market clearing multi-period bonds
exp(QB) = exp(Q) + QCB;

//16. Goods market clearing
exp(Y) = exp(C) + exp(SHOCKG);

//17. Production function
exp(Y) = exp(SHOCKA)*(exp(L)/exp(D))^(1/phi);

//18. Price dispersion
exp(D) = (1-alf)*((1-alf*exp(PI)^(exp(SHOCKT)-1))/(1-alf))
          ^(exp(SHOCKT)*phi/(exp(SHOCKT)-1)) 
         + alf*exp(PI)^(exp(SHOCKT)*phi)*exp(D(-1));


// SHOCKS TO ...
//. Preferences
SHOCKC = rho_c*SHOCKC(-1) - sig_c*e_c;

//. Labour supply
SHOCKL = rho_l*SHOCKL(-1) + sig_l*e_l;

//. Technology
SHOCKA = rho_a*SHOCKA(-1) + sig_a*e_a;

//. 
SHOCKT = (1-rho_t)*log(tet)      + rho_t*SHOCKT(-1) + sig_t*e_t;

//. Government spending
SHOCKG = (1-rho_g)*log(GYSS*YSS) + rho_g*SHOCKG(-1) + sig_g*e_g;

//. Monetary policy (EXPANSIONARY!)
SHOCKN = rho_n*SHOCKN(-1) - sig_n*e_n;

//. Government asset purchases (EXPANSIONARY!)
SHOCKX = rho_x*SHOCKX(-1) - sig_x*e_x;


// FOR PLOTTING
//. Total savings
exp(PSS) = exp(PS)*exp(S);

end;



steady_state_model;                 % all not listed are 0 in SS
PI = log(PISS);
D  = log((1-alf)/(1-alf*PISS^(tet*phi))*((1-alf*PISS^(tet-1))
         /(1-alf))^(tet*phi/(tet-1)));
Y  = log(((1-alf*bet*PISS^(tet-1))/(1-alf*bet*PISS^(tet*phi))
         *(tet*phi)/(tet-1)*exp(D)^psi*(1-GYSS)^del
         *((1-alf*PISS^(tet-1))/(1-alf))^((tet*(phi-1)+1)
         /(tet-1)))^(1/(1-del-phi*(psi+1))));
C  = log((1-GYSS)*exp(Y));
F  = log(exp(Y)*exp(C)^(-del)/(1-alf*bet*PISS^(tet-1)));
K  = log(1/(1-alf*bet*PISS^(tet*phi))*tet*phi/(tet-1)
         *exp(D)^psi*exp(Y)^(phi*(psi+1)));
L  = log(exp(D)*exp(Y)^phi);
W  = log(exp(L)^psi*exp(C)^del);
PS = log(bet/PISS);
QB = log(f*exp(Y));
Q  = QB;
S  = log(PISS/(1-bet)*(exp(C)-tp*exp(W)*exp(L)-(1-tp-GYSS)*exp(Y)));
B  = log(exp(S)-exp(Q)/tau*(1-1/PISS^tau)*PISS/(PISS-1));

// eq.s for PQ and PB only valid when a2=0!
PB = log(a1*exp(PS)*exp(S)/(exp(B)-(1-a1)*gB
                            +(1-a1)*(exp(B)-gB)/(exp(Q)-gQ)*gQ));
PQ = log((1-a1)/a1*(exp(B)-gB)/(exp(Q)-gQ)*exp(PB));

I  = log(1/exp(PB)-1);
IQ = log(0.0110589449073590); % tau=20 -- log(0.034888331506062); % tau=2 % 
SHOCKT = log(tet);
SHOCKG = log(GYSS*exp(Y));
PSS = PS+S;

end;


shocks; 
var e_c = 0;
var e_l = 0;
var e_a = 0;
var e_t = 0;
var e_g = 0;
var e_n = 1;
var e_x = 1;

end;

resid;
check;
steady;
model_diagnostics;


options_.pruning        = 1;
options_.relative_irf   = 0;
options_.nocorr         = 1;
options_.nomoments      = 1;
options_.noprint        = 0;
options_.nograph        = 1;
options_.simul_seed     = 42;
stoch_simul(order=1, periods=1000, irf=20) PI Y I PB PQ PSS B Q S QCB L W;

